import requests
import os
import stat
import json
import getpass
#for making the auth, maybe make its own folder
from .logger import logging as log
from dmc import gettoken
from cachetools import TTLCache

class getConfigPath:
    def __init__(self, fName="config.json"):
        if os.name == "posix":        
            self.path = os.path.join(os.path.expanduser('~'), fName)
        elif os.name == "nt":
            self.path = os.path.join(os.environ['USERPROFILE'], fName)
    @property
    def real_path(self):
        return self.path
class f_check:
    def __init__(self):
        with open(getConfigPath().real_path, 'r') as json_file:
            self.loaded = json.load(json_file)
            self.dump = json.dumps(self.loaded)
            self.loads = json.loads(self.dump)
class auth:
    def __init__(self):
        log.info('Starting Auth process...')
        if f_check().loaded['auth'] == 'DMC':
            log.info('Setting auth headers for DMC......')
            self._headers = {}
            self._headers["X-CENTRIFY-NATIVE-CLIENT"] = 'true'
            self._headers['X-CFY-SRC' ]= 'python'
            try:
                self._headers['Authorization']  = 'Bearer {0}'.format(gettoken(f_check().loaded['scope']))
            except KeyError:
                log.error('Issue with getting DMC scope')
                raise Exception
        elif f_check().loaded['auth'] == 'OAUTH': 
            if 'client_secret' not in f_check().loaded['body']: 
                log.warning("Password Not Saved, Please provide password of Oauth Account or save PW")
                self._pw = getpass.getpass("Please provide Password for Oauth account: {0}\n".format(f_check().loaded['body']['client_id']))
                # Use the auth class to call that instead of this shit
                self.json_d = json.dumps(f_check().loaded)
                self.update = json.loads(self.json_d)
                ########
                self.update['body']['client_secret'] = self._pw
            self._rheaders = {}
            self._rheaders['X-CENTRIFY-NATIVE-CLIENT'] = 'true'
            self._rheaders['Content-Type'] = 'application/x-www-form-urlencoded'
            self._rbody = self.update['body']
            log.info('Oauth URL of app is: {}'.format(self.update['urls']['app_url'])) 
            log.info('Oauth token request Headers are: {}'.format(self._rheaders)) 
            try:
                log.info('Setting auth headers for OAUTH......')
                req = requests.post(url= self.update['urls']['app_url'], headers= self._rheaders, data= self._rbody).json()
            except:
                log.error("Issue getting token")
                log.error("Response: {0}".format(json.dumps(req)))
                raise Exception
            self._headers = {}
            #FIGURE OUT WHY THE FUCK THIS BREAKS THE ENTIRE THING
            #try:
            self._headers["Authorization"] = "Bearer {access_token}".format(**req)
            #except KeyError:
            #    print("wrong PW more than likely")
            #    log.error("Issue setting the headers")
            self._headers["X-CENTRIFY-NATIVE-CLIENT"] = 'true'
        else:
            log.error('An Internal error occurred. Please check the congif file in current users working dir.')
    @property
    def headers(self):
        return self._headers
class Cache:
    def __init__(self):
        self._cache = TTLCache(maxsize=10, ttl=360)
        self._cache['header'] = auth().headers
    @property
    def cached(self):
        return self._cache

#put in mem cache here to instantiate it and have it be called from this file




import requests
import json
import traceback
from .logger import logging as log
from .utility import Cache as c
from .utility import f_check as f

# AT's Work
def boolize(v):
    return {
        "TRUE": True,
        "FALSE": False,
    }.get(v.upper() if hasattr(v,"upper") else "", v)
def sanitizedict(d):
    return {k:boolize(v) for k,v in d.items() if v!= ""}
# AT's Work
def rem_null(args):
    return dict((k, v) for k, v in args.items() if v != None and v != '')
#RedRock Query
class query_request:
    def __init__(self, sql, Debug=False):
        url = "{0}/Redrock/Query".format(f().loaded['urls']['tenant'])
        self._q_headers = c().cached['header']
        try:
            log.info("Starting query...")
            self.query_request = requests.post(url=url, headers=self._q_headers, json={"Script": sql}).json()
        except Exception as e:
            log.error("Internal error occurred. Please note it failed on a Query request.")
            log.error(traceback.print_exc(e))
        self.jsonlist = json.dumps(self.query_request)
        self.parsed_json = (json.loads(self.jsonlist))
        if self.parsed_json['success'] == False:
            log.error("Issue with Query. Dump is: {0}".format(self.jsonlist))
            return None
        log.debug("JSON dump of Query is : {0}".format(self.jsonlist))
        log.info("Finished Query")
        if Debug == True:
            print(json.dumps(self.parsed_json, indent=4, sort_keys=True))
#for other requests
class other_requests:
    def __init__(self, Call, Debug=False, **kwargs):
        Call = '{0}{1}'.format(f().loaded['urls']['tenant'], Call)
        self._r_headers = c().cached['header']
        self.kwargs = kwargs
        self.__dict__.update(**self.kwargs) 
        try:
            log.info("Starting request...")       
            self.other_requests = requests.post(url=Call, headers=self._r_headers, json=self.kwargs).json()
        except Exception as e:
            log.error("Internal error occurred. Please note it failed on an other request")
            log.error(traceback.print_exc(e))
        self.jsonlist = json.dumps(self.other_requests)
        self.parsed_json = (json.loads(self.jsonlist))
        if self.parsed_json['success'] == False:
            log.error("Issue with other request. Dump is: {0}".format(self.jsonlist))
            return None
        log.debug("JSON dump of request is : {0}".format(self.jsonlist))
        log.info("Finished request")
        if Debug == True:
            print(json.dumps(self.parsed_json, indent=4, sort_keys=True))
#make complicated request so that it trasfers complicated objects. This will allow to arrayed dicts and whatnot



import logging
from .utility import f_check as f

try:
    level = logging.getLevelName(f().loaded['debug_level'])
except:
    logging.basicConfig(level = logging.DEBUG, datefmt='%Y-%m-%d %H:%M:%S', \
    format='%(asctime)s %(name)s %(levelname)-8s %(message)s')
else:
    logging.basicConfig(level = level, datefmt='%Y-%m-%d %H:%M:%S', \
        format='%(asctime)s %(name)s %(levelname)-8s %(message)s')



import setuptools
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

thelibFolder = os.path.dirname(os.path.realpath(__file__))
requirementPath = thelibFolder + '/requirements.txt'
install_requires = [] 
if os.path.isfile(requirementPath):
    with open(requirementPath) as f:
        install_requires = f.read().splitlines()

setuptools.setup(
    name="paspymod", 
    version="0.1.6",
    author="Andrew Schilling",
    author_email="andrew.schilling@centrify.com",
    description="Centrify PAS REST API Module to run scripts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ajskrilla/pas_py_mod",
    packages=setuptools.find_packages(include=['paspymod']),    
    install_requires=install_requires,
    python_requires='>=3.6',
)
